title: Vertical rhythm using CSS lh and rlh units
url: https://pawelgrzybek.com/vertical-rhythm-using-css-lh-and-rlh-units/
hash_url: c224a3174ad7a76fae1aaa8d174ec791
archive_date: 2024-04-18
og_image: https://pawelgrzybek.com/vertical-rhythm-using-css-lh-and-rlh-units/og.jpg
description: Vertical rhythm is a design concept that helps to create a harmonious layout by following consistent spacing between elements, typically using the height of a line as a base.
favicon: https://pawelgrzybek.com/icon.svg
language: en_US

<p>Vertical rhythm is a design concept that helps to create a harmonious layout by following consistent spacing between elements, typically using the height of a line as a base. I learned it in my design days when printed media was still a thing<sup id="fnref:1"></sup>. How much the same concept applies to the Web’s fluid nature is debatable, but this post is not about that. To fulfil your curiosity — yes, the primitively simple design of my website follows the vertical rhythm.</p><picture><source srcset="2023-05-16-1.avif" type="image/avif"><source srcset="2023-05-16-1.webp" type="image/webp"><img src="https://pawelgrzybek.com/vertical-rhythm-using-css-lh-and-rlh-units/2023-05-16-1.jpg" alt="Vertical rhythm grid on my website" loading="lazy" decoding="async"></source></source></picture><p>Correctly implementing vertical rhythm is a tedious task. Luckily, the <a href="https://www.w3.org/TR/css-values-4/">CSS Values and Units Module Level 4</a> defines two helpful units that make the job a breeze: <code>lh</code> and <code>rlh</code>, which are equal to the computed <code>line-height</code> of the element on which it is used and the root element, respectively.</p><p>At the time of writing this article, <a href="https://caniuse.com/?search=lh%20unit">the browser support</a> is limited to Safari and Chromium-based browsers. Have a look at this simple example.</p><div class="highlight"><pre tabindex="0" class="chroma"><code class="language-css" data-lang="css"><span class="line"><span class="cl"><span class="nt">html</span> <span class="p">{</span>
</span></span><span class="line"><span class="cl">  <span class="k">font-size</span><span class="p">:</span> <span class="mi">100</span><span class="kt">%</span><span class="p">;</span>
</span></span><span class="line"><span class="cl">  <span class="k">line-height</span><span class="p">:</span> <span class="mf">1.5</span><span class="p">;</span>
</span></span><span class="line"><span class="cl"><span class="p">}</span>
</span></span><span class="line"><span class="cl">
</span></span><span class="line"><span class="cl"><span class="nt">main</span> <span class="p">{</span>
</span></span><span class="line"><span class="cl">  <span class="k">padding</span><span class="p">:</span> <span class="mi">1</span><span class="n">rlh</span><span class="p">;</span> <span class="c">/* 🫶 */</span>
</span></span><span class="line"><span class="cl"><span class="p">}</span>
</span></span></code></pre></div><p>This post is not a tutorial, just a quick appreciation of modern CSS features. Expect more of these because CSS tricks worth writing about are bombarding me daily. Also, it is the first article where I embedded <a href="https://stackblitz.com">StackBlitz</a> demo — it is a great tool. Have a great day, folks 😙</p><div class="share" id="share"><p>Did you like it? Please share it with your friends. Thanks!</p>
